@@ -52,10 +52,12 @@
 
 #include <QtGui/QKeyEvent>
 #include <QtWidgets/QMenu>
+#ifndef QT_NO_PRINTER
 #include <QtPrintSupport/QPageSetupDialog>
 #include <QtPrintSupport/QPrintDialog>
 #include <QtPrintSupport/QPrintPreviewDialog>
 #include <QtPrintSupport/QPrinter>
+#endif
 #include <QtWidgets/QStackedWidget>
 #include <QtWidgets/QTextBrowser>
 #include <QtWidgets/QVBoxLayout>
@@ -401,7 +403,7 @@ void CentralWidget::previousPage()
 void CentralWidget::print()
 {
     TRACE_OBJ
-#ifndef QT_NO_PRINTER
+#if !defined(QT_NO_PRINTER) && !defined(QT_NO_PRINTDIALOG)
     initPrinter();
     QPrintDialog dlg(m_printer, this);
 
@@ -418,7 +420,7 @@ void CentralWidget::print()
 void CentralWidget::pageSetup()
 {
     TRACE_OBJ
-#ifndef QT_NO_PRINTER
+#if !defined(QT_NO_PRINTER) && !defined(QT_NO_PRINTDIALOG)
     initPrinter();
     QPageSetupDialog dlg(m_printer);
     dlg.exec();
@@ -428,7 +430,7 @@ void CentralWidget::pageSetup()
 void CentralWidget::printPreview()
 {
     TRACE_OBJ
-#ifndef QT_NO_PRINTER
+#if !defined(QT_NO_PRINTER) && !defined(QT_NO_PRINTDIALOG)
     initPrinter();
     QPrintPreviewDialog preview(m_printer, this);
     connect(&preview, SIGNAL(paintRequested(QPrinter*)),
@@ -613,6 +615,7 @@ void CentralWidget::initPrinter()
 void CentralWidget::connectSignals(HelpViewer *page)
 {
     TRACE_OBJ
+#if !defined(QT_NO_WEBKIT)
     connect(page, SIGNAL(copyAvailable(bool)), this,
         SIGNAL(copyAvailable(bool)));
     connect(page, SIGNAL(forwardAvailable(bool)), this,
@@ -623,6 +626,9 @@ void CentralWidget::connectSignals(HelpViewer *page)
         SLOT(handleSourceChanged(QUrl)));
     connect(page, SIGNAL(printRequested()), this, SLOT(print()));
     connect(page, SIGNAL(highlighted(QString)), this, SLOT(slotHighlighted(QString)));
+#else
+    Q_UNUSED(page)
+#endif
 }
 
 bool CentralWidget::eventFilter(QObject *object, QEvent *e)
