@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
@@ -74,6 +74,7 @@ namespace {
     const QString VersionKey(QString(QLatin1String("qtVersion%1$$$%2")).
                              arg(QLatin1String(QT_VERSION_STR)));
     const QString ShowTabsKey(QLatin1String("showTabs"));
+    const QString TopicChooserGeometryKey(QLatin1String("TopicChooserGeometry"));
 } // anonymous namespace
 
 class TimeoutForwarder : public QObject
@@ -336,12 +337,6 @@ QString HelpEngineWrapper::error() const
     return d->m_helpEngine->error();
 }
 
-QString HelpEngineWrapper::virtualFolderForNameSpace(const QString &nameSpace) const
-{
-    TRACE_OBJ
-    return d->m_helpEngine->d->nameSpaceVirtualFolderMap.value(nameSpace, QString());
-}
-
 const QStringList HelpEngineWrapper::qtDocInfo(const QString &component) const
 {
     TRACE_OBJ
@@ -730,6 +725,18 @@ bool HelpEngineWrapper::fullTextSearchFallbackEnabled() const
     return CollectionConfiguration::fullTextSearchFallbackEnabled(*d->m_helpEngine);
 }
 
+const QByteArray HelpEngineWrapper::topicChooserGeometry() const
+{
+    TRACE_OBJ
+    return d->m_helpEngine->customValue(TopicChooserGeometryKey).toByteArray();
+}
+
+void HelpEngineWrapper::setTopicChooserGeometry(const QByteArray &geometry)
+{
+    TRACE_OBJ
+    d->m_helpEngine->setCustomValue(TopicChooserGeometryKey, geometry);
+}
+
 // -- TimeoutForwarder
 
 TimeoutForwarder::TimeoutForwarder(const QString &fileName)
