@@ -50,8 +50,6 @@
 #include <QtCore/QString>
 #include <QtCore/QStringList>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -119,6 +117,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif  // QHELPSEARCHENGINE_H
