@@ -46,8 +46,6 @@
 
 #include <QtWidgets/QWidget>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QDesignerFormEditorInterface;
@@ -71,6 +69,4 @@ public Q_SLOTS:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // ABSTRACTACTIONEDITOR_H
