@@ -57,8 +57,6 @@
 #include <QtWidgets/QMessageBox>
 #include <QtWidgets/QFileDialog>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QWidget;
@@ -102,6 +100,4 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // ABSTRACTDIALOGGUI_H
