@@ -48,8 +48,6 @@
 #include <QtCore/QObject>
 #include <QtCore/QScopedPointer>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QDesignerFormEditorInterface;
@@ -157,6 +155,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // ABSTRACTFORMWINDOWMANAGER_H
