@@ -55,8 +55,6 @@
 
 #include <QtDesigner/extension.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QDialog;
@@ -103,6 +101,4 @@ Q_DECLARE_EXTENSION_INTERFACE(QDesignerLanguageExtension, "org.qt-project.Qt.Des
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QDESIGNER_ABTRACT_LANGUAGE_H
