@@ -46,8 +46,6 @@
 
 #include <QtWidgets/QWidget>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QDesignerFormEditorInterface;
@@ -79,6 +77,4 @@ public Q_SLOTS:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // ABSTRACTPROPERTYEDITOR_H
