@@ -46,8 +46,6 @@
 
 #include <QtWidgets/QWidget>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QWidget; // FIXME: fool syncqt
@@ -69,7 +67,5 @@ Q_SIGNALS:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // ABSTRACTFORMEDITOR_H
 
