@@ -46,8 +46,6 @@
 
 #include <QtCore/QVariant>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QString;
@@ -68,6 +66,4 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // ABSTRACTSETTINGS_P_H
