@@ -62,6 +62,7 @@
 #include <QtWidgets/QApplication>
 #include <QtWidgets/QLayout>
 #include <QtWidgets/QUndoStack>
+#include <QtWidgets/QSplitter>
 
 #include <QtWidgets/QFrame>
 #include <QtWidgets/QGroupBox>
@@ -457,6 +458,13 @@ void MorphWidgetCommand::morph(QWidget *before, QWidget *after)
         Q_ASSERT(lh);
         lh->replaceWidget(containingLayout, before, after);
         delete lh;
+    } else if (QSplitter *splitter = qobject_cast<QSplitter *>(parent)) {
+        const int index = splitter->indexOf(before);
+        before->hide();
+        before->setParent(0);
+        splitter->insertWidget(index, after);
+        after->setParent(parent);
+        after->setGeometry(oldGeom);
     } else {
         before->hide();
         before->setParent(0);
