@@ -76,7 +76,6 @@ WidgetDataBaseItem::WidgetDataBaseItem(const QString &name, const QString &group
       m_group(group),
       m_compat(0),
       m_container(0),
-      m_form(0),
       m_custom(0),
       m_promoted(0)
 {
@@ -365,7 +364,6 @@ void WidgetDataBase::loadPlugins()
 {
     typedef QMap<QString, int> NameIndexMap;
     typedef QList<QDesignerWidgetDataBaseItemInterface*> ItemList;
-    typedef QMap<QString, QDesignerWidgetDataBaseItemInterface*> NameItemMap;
     typedef QSet<QString> NameSet;
     // 1) create a map of existing custom classes
     NameIndexMap existingCustomClasses;
