@@ -52,8 +52,6 @@
 #include <QtWidgets/QSizePolicy>
 #include <QtGui/QPalette>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 #if 0
 // pragma for syncqt, don't remove.
@@ -268,6 +266,4 @@ protected:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // ABSTRACTFORMBUILDER_H
