@@ -48,6 +48,7 @@
 #include <QtDesigner/QDesignerPropertyEditorInterface>
 
 #include <QtDesigner/QExtensionManager>
+#include <private/qdesigner_utils_p.h>
 #include <QtCore/QDebug>
 #include <QtCore/QTimer>
 
@@ -99,6 +100,15 @@ bool QAxWidgetPropertySheet::reset(int index)
     return true;
 }
 
+QVariant QAxWidgetPropertySheet::property(int index) const
+{
+    // QTBUG-34592, accessing the 'control' property via meta object system
+    // may cause crashes during loading for some controls.
+    return propertyName(index) == m_controlProperty ?
+        QVariant(axWidget()->control()) :
+        QDesignerPropertySheet::property(index);
+}
+
 void QAxWidgetPropertySheet::setProperty(int index, const QVariant &value)
 {
 
@@ -111,7 +121,9 @@ void QAxWidgetPropertySheet::setProperty(int index, const QVariant &value)
     }
     // Loading forms: Reload
     if (name == m_controlProperty) {
-        const QString clsid = value.toString();
+        const QString clsid = value.canConvert<qdesigner_internal::PropertySheetStringValue>() ?
+            qvariant_cast<qdesigner_internal::PropertySheetStringValue>(value).value() :
+            value.toString();
         if (clsid.isEmpty() || !axWidget()->loadControl(clsid))
             reset(index);
         else
