@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
@@ -281,6 +281,24 @@ static const struct symbol_map_t symbol_map[] = {
     { "Next",          Qt::Key_PageDown },
     { "Help",          Qt::Key_Help },
     { "Pause",         Qt::Key_Pause },
+    { "VolumeDown",    Qt::Key_VolumeDown },
+    { "VolumeUp",      Qt::Key_VolumeUp },
+    { "MediaTogglePlayPause", Qt::Key_MediaTogglePlayPause },
+    { "MediaPlay",     Qt::Key_MediaPlay },
+    { "MediaPause",    Qt::Key_MediaPause },
+    { "MediaStop",     Qt::Key_MediaStop },
+    { "MediaNext",     Qt::Key_MediaNext },
+    { "MediaPrevious", Qt::Key_MediaPrevious },
+    { "AudioForward",  Qt::Key_AudioForward },
+    { "AudioRewind",   Qt::Key_AudioRewind },
+    { "Camera",        Qt::Key_Camera },
+    { "CameraFocus",   Qt::Key_CameraFocus },
+
+    { "Call",             Qt::Key_Call },
+    { "Hangup",           Qt::Key_Hangup },
+    { "ToggleCallHangup", Qt::Key_ToggleCallHangup },
+    { "VoiceDial"       , Qt::Key_VoiceDial },
+    { "LastNumberRedial", Qt::Key_LastNumberRedial },
 
     { "KP_0",        Qt::Key_0 | Qt::KeypadModifier },
     { "KP_1",        Qt::Key_1 | Qt::KeypadModifier },
