@@ -112,7 +112,6 @@ static int yyInPos;
 // The parser maintains the following global variables.
 static QString yyPackage;
 static QStack<Scope*> yyScope;
-static QString yyDefaultContext;
 
 std::ostream &yyMsg(int line = 0)
 {
@@ -164,11 +163,6 @@ static int getToken()
                 }
             }
             switch ( yyIdent.at(0).toLatin1() ) {
-            case 'T':
-                // TR() for when all else fails
-                if ( yyIdent == QLatin1String("TR") )
-                    return Tok_tr;
-                break;
             case 'p':
                 if( yyIdent == QLatin1String("package") )
                     return Tok_Package;
@@ -451,22 +445,22 @@ static const QString context()
              innerClass = true;
          }
      }
-     return context.isEmpty() ? yyDefaultContext : context;
+     return context;
 }
 
 static void recordMessage(
     Translator *tor, const QString &context, const QString &text, const QString &comment,
-    const QString &extracomment, bool plural)
+    const QString &extracomment, bool plural, ConversionData &cd)
 {
     TranslatorMessage msg(
         context, text, comment, QString(),
         yyFileName, yyLineNo, QStringList(),
         TranslatorMessage::Unfinished, plural);
     msg.setExtraComment(extracomment.simplified());
-    tor->extend(msg);
+    tor->extend(msg, cd);
 }
 
-static void parse( Translator *tor )
+static void parse(Translator *tor, ConversionData &cd)
 {
     QString text;
     QString com;
@@ -507,7 +501,7 @@ static void parse( Translator *tor )
                     }
                 }
                 if (!text.isEmpty())
-                    recordMessage(tor, context(), text, com, extracomment, plural);
+                    recordMessage(tor, context(), text, com, extracomment, plural, cd);
             }
             break;
         case Tok_translate:
@@ -536,7 +530,7 @@ static void parse( Translator *tor )
                         }
                     }
                     if (!text.isEmpty())
-                        recordMessage(tor, contextOverride, text, com, extracomment, plural);
+                        recordMessage(tor, contextOverride, text, com, extracomment, plural, cd);
                 }
             }
             break;
@@ -611,7 +605,6 @@ bool loadJava(Translator &translator, const QString &filename, ConversionData &c
         return false;
     }
 
-    yyDefaultContext = cd.m_defaultContext;
     yyInPos = -1;
     yyFileName = filename;
     yyPackage.clear();
@@ -622,12 +615,7 @@ bool loadJava(Translator &translator, const QString &filename, ConversionData &c
     yyParenLineNo = 1;
 
     QTextStream ts(&file);
-    QByteArray codecName;
-    if (!cd.m_codecForSource.isEmpty())
-        codecName = cd.m_codecForSource;
-    else
-        codecName = translator.codecName(); // Just because it should be latin1 already
-    ts.setCodec(QTextCodec::codecForName(codecName));
+    ts.setCodec(QTextCodec::codecForName(cd.m_sourceIsUtf16 ? "UTF-16" : "UTF-8"));
     ts.setAutoDetectUnicode(true);
     yyInStr = ts.readAll();
     yyInPos = 0;
@@ -635,10 +623,7 @@ bool loadJava(Translator &translator, const QString &filename, ConversionData &c
     yyCurLineNo = 1;
     yyParenLineNo = 1;
 
-    parse(&translator);
-
-    // Java uses UTF-16 internally and Jambi makes UTF-8 for tr() purposes of it.
-    translator.setCodecName("UTF-8");
+    parse(&translator, cd);
     return true;
 }
 
