@@ -48,7 +48,6 @@
 #include <QtCore/QDebug>
 #include <QtCore/QMap>
 #include <QtCore/QStringList>
-#include <QtCore/QTextCodec>
 #include <QtCore/QVector>
 
 QT_BEGIN_NAMESPACE
@@ -338,7 +337,6 @@ Translator merge(
     outTor.setLanguageCode(tor.languageCode());
     outTor.setSourceLanguageCode(tor.sourceLanguageCode());
     outTor.setLocationsType(tor.locationsType());
-    outTor.setCodecName(tor.codecName());
 
     /*
       The types of all the messages from the vernacular translator
@@ -353,53 +351,55 @@ Translator merge(
             if (mvi >= 0)
                 m.setComment(virginTor.constMessage(mvi).comment());
         } else {
+            TranslatorMessage::ExtraData extras;
             const TranslatorMessage *mv;
             int mvi = virginTor.find(m);
             if (mvi < 0) {
                 if (!(options & HeuristicSimilarText)) {
                   makeObsolete:
-                    newType = TranslatorMessage::Obsolete;
-                    if (m.type() != TranslatorMessage::Obsolete)
+                    switch (m.type()) {
+                    case TranslatorMessage::Finished:
+                        newType = TranslatorMessage::Vanished;
                         obsoleted++;
+                        break;
+                    case TranslatorMessage::Unfinished:
+                        newType = TranslatorMessage::Obsolete;
+                        obsoleted++;
+                        break;
+                    default:
+                        newType = m.type();
+                        break;
+                    }
                     m.clearReferences();
                 } else {
                     mvi = virginTor.find(m.context(), m.comment(), m.allReferences());
                     if (mvi < 0) {
                         // did not find it in the virgin, mark it as obsolete
                         goto makeObsolete;
-                    } else {
-                        mv = &virginTor.constMessage(mvi);
-                        // Do not just accept it if its on the same line number,
-                        // but different source text.
-                        // Also check if the texts are more or less similar before
-                        // we consider them to represent the same message...
-                        if (getSimilarityScore(m.sourceText(), mv->sourceText()) >= textSimilarityThreshold) {
-                            // It is just slightly modified, assume that it is the same string
-
-                            // Mark it as unfinished. (Since the source text
-                            // was changed it might require re-translating...)
-                            newType = TranslatorMessage::Unfinished;
-                            ++similarTextHeuristicCount;
-                            neww++;
-
-                          outdateSource:
-                            m.setOldSourceText(m.sourceText());
-                            m.setSourceText(mv->sourceText());
-                            const QString &oldpluralsource = m.extra(QLatin1String("po-msgid_plural"));
-                            if (!oldpluralsource.isEmpty()) {
-                                m.setExtra(QLatin1String("po-old_msgid_plural"), oldpluralsource);
-                                m.unsetExtra(QLatin1String("po-msgid_plural"));
-                            }
-                            goto copyAttribs; // Update secondary references
-                        } else {
-                            // The virgin and vernacular sourceTexts are so
-                            // different that we could not find it.
-                            goto makeObsolete;
-                        }
                     }
+                    mv = &virginTor.constMessage(mvi);
+                    // Do not just accept it if its on the same line number,
+                    // but different source text.
+                    // Also check if the texts are more or less similar before
+                    // we consider them to represent the same message...
+                    if (getSimilarityScore(m.sourceText(), mv->sourceText()) < textSimilarityThreshold) {
+                        // The virgin and vernacular sourceTexts are so different that we could not find it
+                        goto makeObsolete;
+                    }
+                    // It is just slightly modified, assume that it is the same string
+
+                    extras = mv->extras();
+
+                    // Mark it as unfinished. (Since the source text
+                    // was changed it might require re-translating...)
+                    newType = TranslatorMessage::Unfinished;
+                    ++similarTextHeuristicCount;
+                    neww++;
+                    goto outdateSource;
                 }
             } else {
                 mv = &virginTor.message(mvi);
+                extras = mv->extras();
                 if (!mv->id().isEmpty()
                     && (mv->context() != m.context()
                         || mv->sourceText() != m.sourceText()
@@ -408,8 +408,14 @@ Translator merge(
                     newType = TranslatorMessage::Unfinished;
                     m.setContext(mv->context());
                     m.setComment(mv->comment());
-                    if (mv->sourceText() != m.sourceText())
-                        goto outdateSource;
+                    if (mv->sourceText() != m.sourceText()) {
+                      outdateSource:
+                        m.setOldSourceText(m.sourceText());
+                        m.setSourceText(mv->sourceText());
+                        const QString &oldpluralsource = m.extra(QLatin1String("po-msgid_plural"));
+                        if (!oldpluralsource.isEmpty())
+                            extras.insert(QLatin1String("po-old_msgid_plural"), oldpluralsource);
+                    }
                 } else {
                     switch (m.type()) {
                     case TranslatorMessage::Finished:
@@ -425,9 +431,14 @@ Translator merge(
                         newType = TranslatorMessage::Unfinished;
                         known++;
                         break;
+                    case TranslatorMessage::Vanished:
+                        newType = TranslatorMessage::Finished;
+                        neww++;
+                        break;
                     case TranslatorMessage::Obsolete:
                         newType = TranslatorMessage::Unfinished;
                         neww++;
+                        break;
                     }
                 }
 
@@ -436,10 +447,9 @@ Translator merge(
                 // This should also enable us to read a file that does not
                 // have the <location> element.
                 // why not use operator=()? Because it overwrites e.g. userData.
-              copyAttribs:
                 m.setReferences(mv->allReferences());
                 m.setPlural(mv->isPlural());
-                m.setUtf8(mv->isUtf8());
+                m.setExtras(extras);
                 m.setExtraComment(mv->extraComment());
                 m.setId(mv->id());
             }
@@ -502,7 +512,8 @@ Translator merge(
                  * will offer them as possible translations.
                  */
                 mv.clearReferences();
-                mv.setType(TranslatorMessage::Obsolete);
+                mv.setType(mv.type() == TranslatorMessage::Finished
+                           ? TranslatorMessage::Vanished : TranslatorMessage::Obsolete);
                 if (options & NoLocations)
                     outTor.append(mv);
                 else
