@@ -55,9 +55,9 @@ void ProFileEvaluator::initialize()
     QMakeEvaluator::initStatics();
 }
 
-ProFileEvaluator::ProFileEvaluator(ProFileGlobals *option, QMakeParser *parser,
+ProFileEvaluator::ProFileEvaluator(ProFileGlobals *option, QMakeParser *parser, QMakeVfs *vfs,
                                    QMakeHandler *handler)
-  : d(new QMakeEvaluator(option, parser, handler))
+  : d(new QMakeEvaluator(option, parser, vfs, handler))
 {
 }
 
@@ -215,6 +215,11 @@ QString ProFileEvaluator::propertyValue(const QString &name) const
     return d->m_option->propertyValue(ProKey(name)).toQString();
 }
 
+QString ProFileEvaluator::resolvedMkSpec() const
+{
+    return d->m_qmakespec;
+}
+
 #ifdef PROEVALUATOR_CUMULATIVE
 void ProFileEvaluator::setCumulative(bool on)
 {
@@ -222,6 +227,21 @@ void ProFileEvaluator::setCumulative(bool on)
 }
 #endif
 
+void ProFileEvaluator::setExtraVars(const QHash<QString, QStringList> &extraVars)
+{
+    ProValueMap map;
+    QHash<QString, QStringList>::const_iterator it = extraVars.constBegin();
+    QHash<QString, QStringList>::const_iterator end = extraVars.constEnd();
+    for ( ; it != end; ++it)
+        map.insert(ProKey(it.key()), ProStringList(it.value()));
+    d->setExtraVars(map);
+}
+
+void ProFileEvaluator::setExtraConfigs(const QStringList &extraConfigs)
+{
+     d->setExtraConfigs(ProStringList(extraConfigs));
+}
+
 void ProFileEvaluator::setOutputDir(const QString &dir)
 {
     d->m_outputDir = dir;
