@@ -52,6 +52,7 @@
 
 QT_BEGIN_NAMESPACE
 
+class QMakeVfs;
 class QMakeParser;
 class QMakeEvaluator;
 class QMakeHandler;
@@ -77,13 +78,16 @@ public:
     // Call this from a concurrency-free context
     static void initialize();
 
-    ProFileEvaluator(ProFileGlobals *option, QMakeParser *parser, QMakeHandler *handler);
+    ProFileEvaluator(ProFileGlobals *option, QMakeParser *parser, QMakeVfs *vfs,
+                     QMakeHandler *handler);
     ~ProFileEvaluator();
 
     ProFileEvaluator::TemplateType templateType() const;
 #ifdef PROEVALUATOR_CUMULATIVE
     void setCumulative(bool on); // Default is false
 #endif
+    void setExtraVars(const QHash<QString, QStringList> &extraVars);
+    void setExtraConfigs(const QStringList &extraConfigs);
     void setOutputDir(const QString &dir); // Default is empty
 
     bool loadNamedSpec(const QString &specDir, bool hostSpec);
@@ -100,6 +104,8 @@ public:
             const ProFile *pro) const;
     QString propertyValue(const QString &val) const;
 
+    QString resolvedMkSpec() const;
+
 private:
     QString sysrootify(const QString &path, const QString &baseDir) const;
 
