@@ -83,12 +83,15 @@ struct QMakeStatics {
     QString strfalse;
     ProKey strCONFIG;
     ProKey strARGS;
+    ProKey strARGC;
     QString strDot;
     QString strDotDot;
     QString strever;
     QString strforever;
     QString strhost_build;
     ProKey strTEMPLATE;
+    ProKey strQMAKE_PLATFORM;
+    ProKey strQMAKESPEC;
 #ifdef PROEVALUATOR_FULL
     ProKey strREQUIRES;
 #endif
