@@ -54,7 +54,7 @@ enum TranslatorSaveMode { SaveEverything, SaveStripped };
 class TranslatorMessage
 {
 public:
-    enum Type { Unfinished, Finished, Obsolete };
+    enum Type { Unfinished, Finished, Vanished, Obsolete };
     typedef QHash<QString, QString> ExtraData;
     class Reference
     {
@@ -129,10 +129,6 @@ public:
 
     Type type() const { return m_type; }
     void setType(Type t) { m_type = t; }
-    bool isUtf8() const { return m_utf8; } // codecForTr override
-    void setUtf8(bool on) { m_utf8 = on; }
-    bool isNonUtf8() const { return m_nonUtf8; } // codecForTr override
-    void setNonUtf8(bool on) { m_nonUtf8 = on; }
     bool isPlural() const { return m_plural; }
     void setPlural(bool isplural) { m_plural = isplural; }
 
@@ -145,7 +141,6 @@ public:
     void setExtras(const ExtraData &extras) { m_extra = extras; }
     void unsetExtra(const QString &key);
 
-    bool needs8Bit() const;
     void dump() const;
 
 private:
@@ -165,8 +160,6 @@ private:
     References  m_extraRefs;
 
     Type m_type;
-    bool m_utf8;
-    bool m_nonUtf8;
     bool m_plural;
 };
 
