@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
@@ -42,15 +42,14 @@
 #include "qdbusviewer.h"
 #include "qdbusmodel.h"
 #include "propertydialog.h"
-
-#include <QtXml/QtXml>
+#include "logviewer.h"
 
 #include <QtWidgets/QTreeWidget>
 #include <QtCore/QStringListModel>
 #include <QtCore/QSortFilterProxyModel>
+#include <QtCore/QMetaProperty>
 #include <QtWidgets/QLineEdit>
 #include <QtWidgets/QListView>
-#include <QtWidgets/QTextBrowser>
 #include <QtWidgets/QAction>
 #include <QtWidgets/QShortcut>
 #include <QtWidgets/QVBoxLayout>
@@ -58,6 +57,9 @@
 #include <QtWidgets/QInputDialog>
 #include <QtWidgets/QMessageBox>
 #include <QtWidgets/QMenu>
+#include <QtDBus/QDBusConnectionInterface>
+#include <QtDBus/QDBusInterface>
+#include <QtDBus/QDBusMetaType>
 
 #include <private/qdbusutil_p.h>
 
@@ -112,7 +114,7 @@ QDBusViewer::QDBusViewer(const QDBusConnection &connection, QWidget *parent)  :
     QSplitter *topSplitter = new QSplitter(Qt::Vertical, this);
     layout->addWidget(topSplitter);
 
-    log = new QTextBrowser;
+    log = new LogViewer;
     connect(log, SIGNAL(anchorClicked(QUrl)), this, SLOT(anchorClicked(QUrl)));
 
     QSplitter *splitter = new QSplitter(topSplitter);
@@ -500,7 +502,7 @@ void QDBusViewer::about()
     box.setText(QString::fromLatin1("<center><img src=\":/qt-project.org/qdbusviewer/images/qdbusviewer-128.png\">"
                 "<h3>%1</h3>"
                 "<p>Version %2</p></center>"
-                "<p>Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).</p>")
+                "<p>Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).</p>")
             .arg(tr("D-Bus Viewer")).arg(QLatin1String(QT_VERSION_STR)));
     box.setWindowTitle(tr("D-Bus Viewer"));
     box.exec();
