@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
@@ -67,13 +67,6 @@
 
 #include <stdlib.h>
 
-#ifndef QT_NO_GSTREAMER
-#include <gst/gst.h>
-#endif
-#ifdef HAVE_PHONON
-#include <phonon/phononnamespace.h>
-#endif
-
 QT_BEGIN_NAMESPACE
 
 // from qapplication.cpp and qapplication_x11.cpp - These are NOT for
@@ -146,14 +139,6 @@ static const char *printer_text = QT_TRANSLATE_NOOP("MainWindow",
 "Qt should search for embeddable font files.  By default, the X "
 "server font path is used.");
 
-static const char *phonon_text = QT_TRANSLATE_NOOP("MainWindow",
-"<p><b><font size+=2>Phonon</font></b></p>"
-"<hr>"
-"<p>Use this tab to configure the Phonon GStreamer multimedia backend. "
-"<p>It is reccommended to leave all settings on \"Auto\" to let "
-"Phonon determine your settings automatically.");
-
-
 QPalette::ColorGroup MainWindow::groupFromIndex(int item)
 {
     switch (item) {
@@ -211,8 +196,6 @@ MainWindow::MainWindow()
     connect(ui->wheelScrollLinesSpinBox, SIGNAL(valueChanged(int)), SLOT(somethingModified()));
     connect(ui->menuEffectCombo, SIGNAL(activated(int)), SLOT(somethingModified()));
     connect(ui->comboEffectCombo, SIGNAL(activated(int)), SLOT(somethingModified()));
-    connect(ui->audiosinkCombo, SIGNAL(activated(int)), SLOT(somethingModified()));
-    connect(ui->videomodeCombo, SIGNAL(activated(int)), SLOT(somethingModified()));
     connect(ui->toolTipEffectCombo, SIGNAL(activated(int)), SLOT(somethingModified()));
     connect(ui->strutWidthSpinBox, SIGNAL(valueChanged(int)), SLOT(somethingModified()));
     connect(ui->strutHeightSpinBox, SIGNAL(valueChanged(int)), SLOT(somethingModified()));
@@ -417,72 +400,6 @@ MainWindow::MainWindow()
     fontpaths = settings.value(QLatin1String("fontPath")).toStringList();
     ui->fontpathListBox->insertItems(0, fontpaths);
 
-    ui->audiosinkCombo->addItem(tr("Auto (default)"), QLatin1String("Auto"));
-    ui->audiosinkCombo->setItemData(ui->audiosinkCombo->findText(tr("Auto (default)")),
-                                    tr("Choose audio output automatically."), Qt::ToolTipRole);
-    ui->audiosinkCombo->addItem(tr("aRts"), QLatin1String("artssink"));
-    ui->audiosinkCombo->setItemData(ui->audiosinkCombo->findText(tr("aRts")),
-                                    tr("Experimental aRts support for GStreamer."),
-                                    Qt::ToolTipRole);
-#ifdef HAVE_PHONON
-    ui->phononVersionLabel->setText(QLatin1String(Phonon::phononVersion()));
-#endif
-#ifndef QT_NO_GSTREAMER
-    if (gst_init_check(0, 0, 0)) {
-        gchar *versionString = gst_version_string();
-        ui->gstVersionLabel->setText(QLatin1String(versionString));
-        g_free(versionString);
-        GList *factoryList = gst_registry_get_feature_list(gst_registry_get_default(),
-                                                           GST_TYPE_ELEMENT_FACTORY);
-        QString name, klass, description;
-        for (GList *iter = g_list_first(factoryList) ; iter != NULL ; iter = g_list_next(iter)) {
-            GstPluginFeature *feature = GST_PLUGIN_FEATURE(iter->data);
-            klass = QLatin1String(gst_element_factory_get_klass(GST_ELEMENT_FACTORY(feature)));
-            if (klass == QLatin1String("Sink/Audio")) {
-                name = QLatin1String(GST_PLUGIN_FEATURE_NAME(feature));
-                if (name == QLatin1String("sfsink"))
-                    continue; // useless to output audio to file when you cannot set the file path
-                else if (name == QLatin1String("autoaudiosink"))
-                    continue; //This is used implicitly from the auto setting
-                GstElement *sink = gst_element_factory_make (qPrintable(name), NULL);
-                if (sink) {
-                    description = QLatin1String(gst_element_factory_get_description(GST_ELEMENT_FACTORY(feature)));
-                    ui->audiosinkCombo->addItem(name, name);
-                    ui->audiosinkCombo->setItemData(ui->audiosinkCombo->findText(name), description,
-                                                    Qt::ToolTipRole);
-                    gst_object_unref (sink);
-                }
-            }
-        }
-        g_list_free(factoryList);
-    }
-#else
-    ui->phononTab->setEnabled(false);
-    ui->phononLabel->setText(tr("Phonon GStreamer backend not available."));
-#endif
-
-    ui->videomodeCombo->addItem(tr("Auto (default)"), QLatin1String("Auto"));
-    ui->videomodeCombo->setItemData(ui->videomodeCombo->findText(tr("Auto (default)")),
-                                    tr("Choose render method automatically"), Qt::ToolTipRole);
-#ifdef Q_WS_X11
-    ui->videomodeCombo->addItem(tr("X11"), QLatin1String("X11"));
-    ui->videomodeCombo->setItemData(ui->videomodeCombo->findText(tr("X11")),
-                                    tr("Use X11 Overlays"), Qt::ToolTipRole);
-#endif
-#ifndef QT_NO_OPENGL
-    ui->videomodeCombo->addItem(tr("OpenGL"), QLatin1String("OpenGL"));
-    ui->videomodeCombo->setItemData(ui->videomodeCombo->findText(tr("OpenGL")),
-                                    tr("Use OpenGL if available"), Qt::ToolTipRole);
-#endif
-    ui->videomodeCombo->addItem(tr("Software"), QLatin1String("Software"));
-    ui->videomodeCombo->setItemData(ui->videomodeCombo->findText(tr("Software")),
-                                    tr("Use simple software rendering"), Qt::ToolTipRole);
-
-    QString audioSink = settings.value(QLatin1String("audiosink"), QLatin1String("Auto")).toString();
-    QString videoMode = settings.value(QLatin1String("videomode"), QLatin1String("Auto")).toString();
-    ui->audiosinkCombo->setCurrentIndex(ui->audiosinkCombo->findData(audioSink));
-    ui->videomodeCombo->setCurrentIndex(ui->videomodeCombo->findData(videoMode));
-
     settings.endGroup(); // Qt
 
     ui->helpView->setText(tr(appearance_text));
@@ -569,13 +486,6 @@ void MainWindow::fileSave()
         settings.setValue(QLatin1String("DefaultInputMethod"), ui->inputMethodCombo->currentText());
 #endif
 
-        QString audioSink = settings.value(QLatin1String("audiosink"), QLatin1String("Auto")).toString();
-        QString videoMode = settings.value(QLatin1String("videomode"), QLatin1String("Auto")).toString();
-        settings.setValue(QLatin1String("audiosink"),
-                          ui->audiosinkCombo->itemData(ui->audiosinkCombo->currentIndex()));
-        settings.setValue(QLatin1String("videomode"),
-                          ui->videomodeCombo->itemData(ui->videomodeCombo->currentIndex()));
-
         QStringList effects;
         if (ui->effectsCheckBox->isChecked()) {
             effects << QLatin1String("general");
@@ -752,7 +662,7 @@ void MainWindow::removeSubstitute()
     if (row > ui->substitutionsListBox->count())
         row = ui->substitutionsListBox->count() - 1;
     ui->substitutionsListBox->setCurrentRow(row);
-    QFont::removeSubstitution(ui->familySubstitutionCombo->currentText());
+    QFont::removeSubstitutions(ui->familySubstitutionCombo->currentText());
     QFont::insertSubstitutions(ui->familySubstitutionCombo->currentText(), subs);
     setModified(true);
 }
@@ -791,7 +701,7 @@ void MainWindow::downSubstitute()
     ui->substitutionsListBox->clear();
     ui->substitutionsListBox->insertItems(0, subs);
     ui->substitutionsListBox->setCurrentRow(row + 1);
-    QFont::removeSubstitution(ui->familySubstitutionCombo->currentText());
+    QFont::removeSubstitutions(ui->familySubstitutionCombo->currentText());
     QFont::insertSubstitutions(ui->familySubstitutionCombo->currentText(), subs);
     setModified(true);
 }
@@ -809,7 +719,7 @@ void MainWindow::upSubstitute()
     ui->substitutionsListBox->clear();
     ui->substitutionsListBox->insertItems(0, subs);
     ui->substitutionsListBox->setCurrentRow(row - 1);
-    QFont::removeSubstitution(ui->familySubstitutionCombo->currentText());
+    QFont::removeSubstitutions(ui->familySubstitutionCombo->currentText());
     QFont::insertSubstitutions(ui->familySubstitutionCombo->currentText(), subs);
     setModified(true);
 }
@@ -902,7 +812,7 @@ void MainWindow::helpAbout()
     QMessageBox box(this);
     box.setText(tr("<h3>%1</h3>"
                    "<br/>Version %2"
-                   "<br/><br/>Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).")
+                   "<br/><br/>Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).")
                    .arg(tr("Qt Configuration")).arg(QLatin1String(QT_VERSION_STR)));
     box.setWindowTitle(tr("Qt Configuration"));
     box.setIcon(QMessageBox::NoIcon);
@@ -925,8 +835,6 @@ void MainWindow::pageChanged(int pageNumber)
         ui->helpView->setText(tr(font_text));
     else if (page == ui->printerTab)
         ui->helpView->setText(tr(printer_text));
-    else if (page == ui->phononTab)
-        ui->helpView->setText(tr(phonon_text));
 }
 
 void MainWindow::closeEvent(QCloseEvent *e)
